<?php

/*
 * Questo programma implementa un algoritmo di criptazione e decriptazione basato sul cifrario di Cesare.
 * È distribuito con licenza GNU General Public License v3.0 o successiva.
 *
 * @author Marco Malvicini
 * @copyright Copyright (c) Marco Malvicini
 * @license https://www.gnu.org/licenses/gpl-3.0.html GNU General Public License v3.0
 *
 * ISTRUZIONI PER L'UTILIZZO:
 *		L'algoritmo è un'evoluzione del cifrario di Cesare e serve a criptare e decriptare stringhe alfanumeriche senza caratteri speciali e senza spazi
 *		Indicato per creare codici alfanumerici criptati da esporre sul web
 *
 * 		get_ctl_char_pattern()
 * 		+ Restituisce il pattern predefinito che contiene i caratteri di controllo.
 * 			- Il numero di caratteri deve essere uguale al numero di elementi(pattern fissi predefiniti) definiti in get_array_pattern()
 * 			- Non sono consentiti caratteri uguali. E' possibile utilizzare due lettere uguali ma una deve essere maiuscola e l'altra minuscola
 *			- Esempio: Se il numero di pattern in get_array_pattern() è 22 la stringa in get_ctl_char_pattern() dovrà avere 22 caratteri alfanumerici diversi.
 *			- '1234567890abcdefghimno' oppure 'qwertyuiop3k5n6m7vVQ0M' oppure 'mMnNbB7651023fFghHjJkz' ecc.
 *
 *		get_array_pattern()
 *		+ Restituisce un array di pattern predefiniti(stringhe fisse) che comprende tutti i caratteri dell'alfabeto (maiuscoli e minuscoli) e tutti i numeri
 *		SENZA RIPETIZIONI!!
 *			- Il numero di caratteri della chiave di controllo in get_ctl_char_pattern() DEVE essere uguale al numero di pattern presenti in get_array_pattern()
 *			- Esempio: 
 *			$array_pattern = array(
 *				'iLAKoRDFHju75G69sx40S3zvQOwMNptdqg1lekChrXyYTJ2E8mancWZfUIBPbV',  // 1
 *					'roE8v43VdLQJwIH0B7F2Rn9Gp5UgaN6CexZflkMt1XPizubmOTyDhYWKscjqSA'); // 2
 *				return $array_pattern[$index];
 *			}
 *			In questo caso abbiamo 2 pattern, pertanto la chiave con i caratteri di controllo in get_ctl_char_pattern() avrà 2 caratteri, es. 'we' oppure '2N' e così via
 *
 *		encrypt_tk($stringa, $chiave) e decrypt_tk($stringaCriptata, $chiave)
 *		+ Le funzioni criptatura e decriptatura passando la $stringa e la $chiave
 *			- ATTENZIONE: La $chiave è una stringa numerica esempio: '123' oppure '9127348' ecc.
 *			- la $chiave può avere lunghezze diverse e deve contenere caratteri numerici senza separatori.
 *			- la sicurezza e la velocità dell'algoritmo di criptazione varia in funzione della lunghezza della $chiave e al numero di pattern fissi presenti in get_array_pattern()
 *
 *
 */

function get_ctl_char_pattern(){
	// centralizziamo e definiamo il pattern che servirà per ricavare il carattere di controllo che determina
	// la posizione del pattarn principale in array_pattern[]
	// NB: $ctl_char_pattern deve essere un codice alfanumerico di caratteri non ripetuti (anche uguali ma minuscoli e maiuscoli).
	// Il numero di elemanti deve essere uguale al numero di pattern presenti in get_array_pattern()
	// ES: $ctl_char_pattern = '1234567890abcdefghimno'(22 elementi), array_pattern dovrà avere 22 pattern
	$ctl_char_pattern = '1234567890abcdefghimno';
	return $ctl_char_pattern;
}

function get_array_pattern($index = 0) {
	// centralizziamo e definiamo i pattern casuali fissi composti da caratteri alfanumerici non ripetuti (con maiuscole e minuscole)
	// per consentire la codifica/decodifica, ogni pattern deve contenere tutte le lettere dell'alfabeto (maiuscole e minuscole) e tutti i numeri da 0 a 9
	// NB: Il numero di elementi di $array_pattern deve essere uguale al numero di elementi di $ctl_char_pattern
	
    $array_pattern = array(
        'iLAKoRDFHju75G69sx40S3zvQOwMNptdqg1lekChrXyYTJ2E8mancWZfUIBPbV',
        'roE8v43VdLQJwIH0B7F2Rn9Gp5UgaN6CexZflkMt1XPizubmOTyDhYWKscjqSA',
        'jSzVpdJBF8eLhTZr0gHbPInKsx2tYwkW1A7vf9oa4uXQUDNyE6GcRli5Om3qCM',
        'tFafQ2hc8CGNIU9XBM0AsuiJzKPdxnpRY5gbjVZky36OreWSD1TELHq4mwov7l',
        '68VmzLlXtNsYpFwjD2aOSWIH0crPf5MeZ4KbGu1xoQnkJTvEdB7RhCg9yiA3qU',
        'xY7AvzZG5j4Hub29KgcPWdtOLSq6IX38sBy1QVmwEiCeRaoNUTMlr0pnJkhfDF',
        'ib15IphWKqRauxmsjflVo4zJ0c8ZSwgCLtyHYU67DAMTPQEFGvnNder9OBX23k',
        'voag6eQ45kswChbWmBZSxGOV08uNI29jlFMAdKyPEXn7D1HLf3zRpqiJcYtTrU',
        'xTCoIGry5wPWpBg9qhzE4t6e1vNiYR37JcXSKnZmDAabkOU0Hls2dfFL8MujVQ',
        'xF3jYfI6HdJiSaNTBrvDpu0U14eX7ZcCytgn8kRq9wLPG5AK2hWEzbsomMQlOV',
        '0Sn6fJet41hgN3aPkZbFDTc2sr9ulUwM8OyCWEdpzj5GVLAmHQq7YRviKXIBxo',
        '4TujZpq2MeO6nxVg9BrLkdwy3zUI0YSsbJN1cR78lAWGHQXCoPaFKEmvhfitD5',
        'JRCLKI0QFa59GAXYwzM6cNP4iuTDdUsymfHZnOb7qh2vWSkorE1tBg3p8Vjelx',
        'hiw1RKr024bcVSuZGTmBAafMjYsU938qzXvCo5ekNldWDEHtxOgQIpFP67ynJL',
        'fksWCza9Rjmhg3pc7rGIvSiloEQ0FZ6dPTq8KJBt1yxuw5ONA4HbDXeVMYUnL2',
        'fJIZ7ALkwFmqSPhp96xQ4zrtEay1KnbXvdWRciu2GCD5O8gsT0U3olHYeBMVjN',
        'sfA5dRCbByhYKLuP410N9reDJqE7tzv3gF8ZcoTnjpX2QIkxiSOa6wmMGWUVlH',
        'CtZnQmSdEDf8JgypbAkYG04UVhTFRjwcqs51IBuMHeXairLlox3v97NW6z2POK',
        'N6310QtcxDh7TerWOB8PpjCS9KsZlLMoEXGvmi4kJzHUqbdgRYFAwafuVyIn25',
        'ZC0Pvb6uQK2D1RYOFIfNMaWGBieVS8rJLmo5Tntlzs7H4U9jkwEpxcX3gyqhdA',
        'KFH04pETuyzhU3Agk5DvrWx8jmCJ6ciQYfw2tqRlVBGdeZIaP17LsObSo9NnMX',
        'yNUJDf4rmbCTuAqtFBX6oPYpEOzeS7csvwHMgQ0ij25hLkaZ93Il1RndWKVxG8'
    );
    return $array_pattern[$index];
}

function encrypt_tk($stringa, $chiave) {
	// $stringa => Stringa da criptare
	// $chiave => Chiave numerica
    $ctl_char_pattern = get_ctl_char_pattern();
    $pattern_index = mt_rand(0, strlen($ctl_char_pattern)-1);	//ricavo l'indice del pattern principale scegliendone uno a caso tra quelli predefiniti
    $ctl_char = $ctl_char_pattern[$pattern_index];	// ricavo il carattere di controllo che servirà per la decodifica
    $array_pattern = get_array_pattern($pattern_index); // ricavo la stringa di codifica/decodifica (il pattern principale selezionato a caso)
    $lunghezzaArray = strlen($array_pattern);
    $lunghezzaChiave = strlen($chiave);
    $stringaCriptata = '';
	
	// Scorro la stringa da criptare, il pattern e mi sposto di n caratteri (ogni ciclo) in base alla chiave numerica passata
    for ($i = 0; $i < strlen($stringa); $i++) {
        $posizione = (strpos($array_pattern, $stringa[$i]) + intval($chiave[$i % $lunghezzaChiave])) % $lunghezzaArray;
        $stringaCriptata .= $array_pattern[$posizione];
    }
	// Restituisco la stringa criptata e aggiungo all'inizio il carattere di controllo.
    return $ctl_char . $stringaCriptata;
}

function decrypt_tk($stringaCriptata, $chiave) {
    // ricavo il pattern con i caratteri di controllo che serviranno ad identificare 
	// l'indice del pattern fisso da utilizzare per la decriptazione
	$ctl_char_pattern = get_ctl_char_pattern(); 
	// il primo carattere della stringa da decriptare, 
	// corrisponde all'indice da utilizzare per determinare il pattern fisso da usare
    $ctl_char = $stringaCriptata[0];	
    $ctl_char_pos = strpos($ctl_char_pattern, $ctl_char);	// la posizione del carattere di controllo nel pattern di controllo, rappresenta l'indice del pattern fisso da usare
    $array_pattern = get_array_pattern($ctl_char_pos); // ricavo il pattern fisso per la decriptazione passando l'indice del carattere di controllo
    $lunghezzaArray = strlen($array_pattern);
    $lunghezzaChiave = strlen($chiave);
    $stringaDecriptata = '';
    $stringaCriptata = substr($stringaCriptata, 1); // stringa da decriptare (pulita) senza il carattere di controllo.
	
	// come per la criptazione, scorro la tringa criptata e cerco il carattere corrispondente l'array fisso e la chiave per decriptare la stringa
    for ($i = 0; $i < strlen($stringaCriptata); $i++) {
        $posizione = (strpos($array_pattern, $stringaCriptata[$i]) - intval($chiave[$i % $lunghezzaChiave]) + $lunghezzaArray) % $lunghezzaArray;
        $stringaDecriptata .= $array_pattern[$posizione];
    }

    return $stringaDecriptata;
}

// Test delle funzioni
$plaintext = "CiaovirgolaComplimentiFunzionaPuntoEsclamativo";
$key = "12371";

$text1 = encrypt_tk($plaintext, $key);
echo "Cifrato: $text1\n";

$decrypted = decrypt_tk($text1, $key);
echo "Decifrato: $decrypted\n";

?>


